import { parseFromString } from "dom-parser";
import { BaseHelper, VideoHelperError } from "./base.js";
import Logger from "@vot.js/shared/utils/logger";
import { proxyMedia } from "@vot.js/shared/utils/utils";
export default class YandexDiskHelper extends BaseHelper {
    API_ORIGIN = this.origin ?? "https://disk.yandex.ru";
    INLINE_PREFIX = "/i/";
    DISK_PREFIX = "/d/";
    clearTitle(title) {
        return title.replace(/(\.[^.]+)$/, "");
    }
    getBodyHash(fileHash, sk) {
        const data = JSON.stringify({
            hash: fileHash,
            sk,
        });
        return encodeURIComponent(data);
    }
    async fetchList(dirHash, sk) {
        const body = this.getBodyHash(dirHash, sk);
        const res = await this.fetch(this.API_ORIGIN + "/public/api/fetch-list", {
            method: "POST",
            body,
        });
        const data = (await res.json());
        if (Object.hasOwn(data, "error")) {
            throw new VideoHelperError("Failed to fetch folder list");
        }
        return data.resources;
    }
    async getDownloadUrl(fileHash, sk) {
        const body = this.getBodyHash(fileHash, sk);
        const res = await this.fetch(this.API_ORIGIN + "/public/api/download-url", {
            method: "POST",
            body,
        });
        const data = (await res.json());
        if (data.error) {
            throw new VideoHelperError("Failed to get download url");
        }
        return data.data.url;
    }
    async getDiskVideoData(videoId) {
        try {
            const res = await this.fetch(this.API_ORIGIN + videoId, {
                headers: {
                    Origin: this.API_ORIGIN,
                    Referer: this.API_ORIGIN + "/client/disk",
                },
            });
            if (res.status !== 200) {
                throw new VideoHelperError("Failed to fetch document");
            }
            const content = await res.text();
            const doc = parseFromString(content.replace(/<!DOCTYPE html>/i, ""));
            const prefetchEl = doc.getElementById("store-prefetch");
            if (!prefetchEl) {
                const reason = doc.getElementById("checkbox-captcha-form")
                    ? "page contains Captcha"
                    : "no prefetch data on page";
                throw new VideoHelperError(`Failed to get prefetch data. Reason: ${reason}`);
            }
            const resourcePaths = videoId.split("/").slice(3);
            if (!resourcePaths.length) {
                throw new VideoHelperError("Failed to find video file path");
            }
            const data = JSON.parse(prefetchEl.textContent);
            const { resources, rootResourceId, environment: { sk }, } = data;
            const rootResource = resources[rootResourceId];
            const resourcePathsLastIdx = resourcePaths.length - 1;
            const resourcePath = resourcePaths
                .filter((_, idx) => idx !== resourcePathsLastIdx)
                .join("/");
            let resourcesList = Object.values(resources);
            if (resourcePath.includes("/")) {
                resourcesList = await this.fetchList(`${rootResource.hash}:/${resourcePath}`, sk);
            }
            const resource = resourcesList.find((resource) => resource.name === resourcePaths[resourcePathsLastIdx]);
            if (!resource) {
                throw new VideoHelperError("Failed to find resource");
            }
            if (resource && resource.type === "dir") {
                throw new VideoHelperError("Path is dir, but expected file");
            }
            const { meta: { short_url, mediatype, videoDuration }, path, name, } = resource;
            if (mediatype !== "video") {
                throw new VideoHelperError("Resource isn't a video");
            }
            const title = this.clearTitle(name);
            const duration = Math.round(videoDuration / 1000);
            if (short_url) {
                return {
                    url: short_url,
                    duration,
                    title,
                };
            }
            const downloadUrl = await this.getDownloadUrl(path, sk);
            return {
                url: proxyMedia(new URL(downloadUrl)),
                duration,
                title,
            };
        }
        catch (err) {
            Logger.warn(`Failed to get yandex disk video data by disk video ID: ${videoId}, because ${err.message}. Using clear link instead...`);
            return {
                url: this.service.url + videoId.slice(1),
            };
        }
    }
    async getVideoData(videoId) {
        if (videoId.startsWith(this.INLINE_PREFIX) ||
            /^\/d\/([^/]+)$/.exec(videoId)) {
            return {
                url: this.service.url + videoId.slice(1),
            };
        }
        videoId = decodeURIComponent(videoId);
        return await this.getDiskVideoData(videoId);
    }
    async getVideoId(url) {
        const fileId = /\/i\/([^/]+)/.exec(url.pathname)?.[0];
        if (fileId) {
            return fileId;
        }
        return /\/d\/([^/]+)/.exec(url.pathname) ? url.pathname : undefined;
    }
}
